__title__ = 'flow-network'
__description__ = 'Flow Network Python Library'
__url__ = 'https://github.com/LucienShui/flow-network'
__version_info__ = (0, 1, 12)
__version__ = '.'.join(map(str, __version_info__))
__build__ = eval(f"0x{''.join(map(lambda x: f'{int(x):02d}', __version_info__))}")
__author__ = 'Lucien Shui'
__author_email__ = 'lucien@lucien.ink'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2020 Lucien Shui'



from __future__ import absolute_import, print_function

import typing
from queue import Queue

INF = 0x3f3f3f3f


class Edge:

    def __init__(self, _next: int, u: int, v: int, flow: int, cost: int):
        self.next_idx = _next
        self.u = u
        self.v = v
        self.flow = flow
        self.cost = cost


class Graph:

    def __init__(self, n: int):
        self.n = n
        self.cnt = 0
        self.head: typing.List[int] = [-1] * n
        self.edges: typing.List[Edge] = []

    def __add_edge(self, u: int, v: int, flow: int, cost: int = 0):
        self.edges.append(Edge(self.head[u], u, v, flow, cost))
        self.head[u] = self.cnt
        self.cnt += 1

    def add_edge(self, u: int, v: int, flow: int, cost: int = 0):
        self.__add_edge(u, v, flow, cost)
        self.__add_edge(v, u, 0, -cost)


class PyMinimumCostFlow:

    def __init__(self, n: int):
        self.n: int = n
        self.graph: Graph = Graph(n)

        self.clk: int = 0

        self.dist: typing.List[int] = ...
        self.pre: typing.List[int] = [0] * n
        self.low: typing.List[int] = [0] * n
        self.vis: typing.List[int] = [0] * n

        self._algorithm_name: str = 'minimum_cost_flow'

    def __bfs(self, s: int, t: int) -> bool:
        self.clk += 1
        self.vis[s] = self.clk
        self.low[s] = INF
        self.dist = [INF] * self.n
        self.dist[s] = 0

        que: Queue = Queue()
        que.put(s)

        while not que.empty():
            u = que.get()
            self.vis[u] = -1

            i = self.graph.head[u]
            while i != -1:
                v = self.graph.edges[i].v
                flow = self.graph.edges[i].flow
                cost = self.graph.edges[i].cost

                if self.dist[v] > self.dist[u] + cost and flow > 0:
                    self.dist[v] = self.dist[u] + cost
                    self.pre[v] = i
                    self.low[v] = min(self.low[u], flow)

                    if self.vis[v] != self.clk:
                        que.put(v)
                        self.vis[v] = self.clk

                i = self.graph.edges[i].next_idx

        return self.dist[t] < INF

    def run(self, s: int, t: int) -> (int, int):
        """
        inference
        :param s: source point's index
        :param t: target point's index
        :return: flow, cost
        """

        flow = cost = 0

        while self.__bfs(s, t):
            flow += self.low[t]
            cost += self.low[t] * self.dist[t]
            u = t
            while u != s:
                self.graph.edges[self.pre[u]].flow -= self.low[t]
                self.graph.edges[self.pre[u] ^ 1].flow += self.low[t]

                u = self.graph.edges[self.pre[u]].u

        return flow, cost



from .core import CBaseNetwork, CMinimumCostFlow
from .pycore import PyMinimumCostFlow
from .edges import EdgeWithCost
from .core import CMaximumFlow
from .edges import Edge

import typing


class NetWork:

    def __init__(self, n: int, algorithm_name: str, c_backend, python_backend=None, backend: str = 'c'):
        super().__init__()

        self._algorithm_name = algorithm_name
        self._n: int = n

        if backend.lower() == 'python' and python_backend is not None:
            self._obj: CBaseNetwork = python_backend(n)
        else:
            self._obj: CBaseNetwork = c_backend(n)

        self.edges: typing.List[Edge] = ...

    def _run(self, s: int, t: int) -> [typing.Tuple, int]:
        """
        inference
        :param s: source point's index
        :param t: target point's index
        :return: tuple or int
        """
        result = self._obj.run(s, t)

        for i in range(0, len(self._obj.graph.edges), 2):
            self.edges[i >> 1].flow = self._obj.graph.edges[i ^ 1].flow

        return result

    def _add_edge(self, u: int, v: int, *args) -> None:

        if not 1 <= len(args) <= 2:
            raise AssertionError('length of args must >= 1 and <= 2')

        for arg in (u, v) + args:
            if not isinstance(arg, int):
                raise AssertionError(f'every arg should be type of int, got {type(arg)}')

        for node in ['u', 'v']:
            if eval(node) < 0:
                raise AssertionError(f'index of {node} is {eval(node)}, which should be greater or equal to 0')

            if eval(node) >= self._n:
                raise AssertionError(f'index of {node} is {eval(node)}, which should be less than n = {self._n}')

        self._obj.graph.add_edge(u, v, *args)

    def summary(self, line_length: int = 32, print_fn=print):
        vertex_cnt: int = self._n
        edge_cnt: int = len(self.edges)

        print_fn(f'''{'=' * line_length}
{' '.join([each.capitalize() for each in self._algorithm_name.split('_')])}
{'-' * line_length}
Number of vertices: {vertex_cnt}
Number of edges: {edge_cnt}
{"=" * line_length}''')

    def extract_graph(self, print_fn=print):
        print_fn('\n'.join(
            [
                'u -> v, flow / capacity' + ', cost' if isinstance(self.edges[0], EdgeWithCost) else ''
            ] + [
                str(edge) for edge in self.edges
            ]))


class MaximumFlow(NetWork):

    def __init__(self, n: int, backend: str = 'c'):
        super().__init__(n, 'flow_network', CMaximumFlow, None, backend)
        self.edges: typing.List[Edge] = []
        self._obj: CMaximumFlow = CMaximumFlow(n)

    def add_edge(self, u: int, v: int, flow: int) -> None:
        """
        add edge from u to v with flow and cost
        :param u: point's index
        :param v: point's index
        :param flow: edge capacity
        :return: None
        """
        self._add_edge(u, v, flow)
        self.edges.append(Edge(u, v, flow))

    def run(self, s: int, t: int) -> int:
        """
        inference
        :param s: source point's index
        :param t: target point's index
        :return: flow
        """
        return self._run(s, t)


class MinimumCostFlow(NetWork):

    def __init__(self, n: int, backend: str = 'c'):
        super().__init__(n, 'minimum_cost_flow', CMinimumCostFlow, PyMinimumCostFlow, backend)
        self.edges: typing.List[EdgeWithCost] = []

    def add_edge(self, u: int, v: int, flow: int, cost: int) -> None:
        """
        add edge from u to v with flow and cost
        :param u: point's index
        :param v: point's index
        :param flow: edge capacity
        :param cost: cost for cutting an edge
        :return: None
        """
        self._add_edge(u, v, flow, cost)
        self.edges.append(EdgeWithCost(u, v, flow, cost))

    def run(self, s: int, t: int) -> (int, int):
        """
        inference
        :param s: source point's index
        :param t: target point's index
        :return: flow, cost
        """
        flow, cost = self._run(s, t)
        return flow, cost



from __future__ import absolute_import, print_function

from .network import MaximumFlow, MinimumCostFlow

from .__version__ import __title__, __description__, __url__
from .__version__ import __version__, __build__, __author__
from .__version__ import __author_email__, __license__, __copyright__



class Edge(object):
    def __init__(self, u: int, v: int, capacity: int, flow: int = 0):
        self.u: int = u
        self.v: int = v
        self.capacity: int = capacity
        self.flow: int = flow

    def __str__(self):
        return f'{self.u} -> {self.v}, {self.flow} / {self.capacity}'


class EdgeWithCost(Edge):
    def __init__(self, u: int, v: int, capacity: int, cost: int, flow: int = 0):
        super().__init__(u, v, capacity, flow)
        self.cost: int = cost

    def __str__(self):
        return f'{super().__str__()}, {self.cost}'



try:
    from .core import MaximumFlow as CMaximumFlow
    from .core import MinimumCostFlow as CMinimumCostFlow
    from .core import BaseNetwork as CBaseNetwork
except Exception as e:
    import sys

    """
    在调用 setup.py 的时候需要读取 flow_network.__version__
    于是 flow_network.__init__.py 会被同时调用，然后一直执行到这里
    但在调用 setup.py 的时候 core.i 还没被编译成 _core
    所以在这里 mock 一些 _core 里面的对象，否则会报错
    """
    pattern: str = 'setup.py'

    if [arg for arg in sys.argv if ~arg.find(pattern)]:
        CMaximumFlow = CMinimumCostFlow = CBaseNetwork = object
    else:
        raise e



from __future__ import absolute_import, print_function

import os

from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from shutil import move

from flow_network.__version__ import __title__, __description__, __url__
from flow_network.__version__ import __version__, __author__
from flow_network.__version__ import __author_email__


def readme(filename: str = 'README.md') -> str:
    """
    read the contents of your README.md file
    :return: contents of README.md
    """

    this_directory = os.path.abspath(os.path.dirname(__file__))

    with open(os.path.join(this_directory, filename), encoding='utf-8') as file:
        return file.read()


class CustomExtension(Extension):
    def __init__(self, name, sources, base_dir, *args, **kw):
        super().__init__(name, sources, *args, **kw)
        self.base_dir = base_dir


class CustomBuild(build_ext):
    def build_extension(self, ext: CustomExtension):
        if isinstance(ext, CustomExtension):
            ext.sources = [os.path.join(ext.base_dir, each) for each in ext.sources]

        super().build_extension(ext)

        if isinstance(ext, CustomExtension):
            raw_output = self.get_ext_fullpath(ext.name)

            filename = os.path.basename(raw_output)
            pathname = os.path.dirname(raw_output)

            output = os.path.join(pathname, ext.base_dir, filename)

            move(raw_output, output)


if __name__ == '__main__':
    custom_extension = CustomExtension(name='_core',
                                       sources=['graph.cpp', 'base_network.cpp', 'maximum_flow.cpp',
                                                'minimum_cost_flow.cpp', 'core_wrap.cxx'],
                                       base_dir='flow_network/core',
                                       extra_compile_args=['-std=c++11'])

    setup(
        name=__title__,
        version=__version__,
        author=__author__,
        author_email=__author_email__,
        url=__url__,
        description=__description__,
        packages=find_packages(),
        ext_modules=[custom_extension],
        cmdclass={'build_ext': CustomBuild},
        long_description=readme(),
        long_description_content_type='text/markdown'
    )



from __future__ import absolute_import, print_function
import unittest

from .network import MaximumFlow, MinimumCostFlow


class FlowNetworkTestCase(unittest.TestCase):

    def maximum_flow_test(self, backend: str = 'c'):
        maximum_flow = MaximumFlow(5, backend)

        # u, v, flow
        edges = [
            (0, 1, 1),
            (1, 2, 1),
            (2, 4, 2),
            (1, 3, 2),
            (3, 4, 1)
        ]

        for u, v, flow in edges:
            maximum_flow.add_edge(u, v, flow)

        maximum_flow.summary()

        self.assertEqual(1, maximum_flow.run(0, 4))

        maximum_flow.extract_graph()

        expected_flow = [1, 0, 0, 1, 1]

        self.assertEqual(expected_flow, [edge.flow for edge in maximum_flow.edges])

    def minimum_cost_flow_test(self, backend: str = 'c'):
        minimum_cost_flow = MinimumCostFlow(5, backend)

        # u, v, flow, cost
        edges = [
            (0, 1, 1, 1),
            (1, 2, 1, 1),
            (2, 4, 2, 2),
            (1, 3, 2, 2),
            (3, 4, 1, 4)
        ]

        for u, v, flow, cost in edges:
            minimum_cost_flow.add_edge(u, v, flow, cost)

        minimum_cost_flow.summary()

        flow, cost = minimum_cost_flow.run(0, 4)

        minimum_cost_flow.extract_graph()

        self.assertEqual(1, flow)
        self.assertEqual(4, cost)

        expect_flow = [1, 1, 1, 0, 0]

        self.assertEqual(expect_flow, [edge.flow for edge in minimum_cost_flow.edges])

    def test_flow_network(self):
        for backend in ['c', 'python']:
            self.maximum_flow_test(backend)
            self.minimum_cost_flow_test(backend)


if __name__ == '__main__':
    unittest.main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_core')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_core')
    _core = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_core', [dirname(__file__)])
        except ImportError:
            import _core
            return _core
        try:
            _mod = imp.load_module('_core', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _core = swig_import_helper()
    del swig_import_helper
else:
    import _core
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _core.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self):
        return _core.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _core.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _core.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _core.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _core.SwigPyIterator_equal(self, x)

    def copy(self):
        return _core.SwigPyIterator_copy(self)

    def next(self):
        return _core.SwigPyIterator_next(self)

    def __next__(self):
        return _core.SwigPyIterator___next__(self)

    def previous(self):
        return _core.SwigPyIterator_previous(self)

    def advance(self, n):
        return _core.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _core.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _core.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _core.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _core.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _core.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _core.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _core.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class EdgeVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, EdgeVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, EdgeVector, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _core.EdgeVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _core.EdgeVector___nonzero__(self)

    def __bool__(self):
        return _core.EdgeVector___bool__(self)

    def __len__(self):
        return _core.EdgeVector___len__(self)

    def __getslice__(self, i, j):
        return _core.EdgeVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _core.EdgeVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _core.EdgeVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _core.EdgeVector___delitem__(self, *args)

    def __getitem__(self, *args):
        return _core.EdgeVector___getitem__(self, *args)

    def __setitem__(self, *args):
        return _core.EdgeVector___setitem__(self, *args)

    def pop(self):
        return _core.EdgeVector_pop(self)

    def append(self, x):
        return _core.EdgeVector_append(self, x)

    def empty(self):
        return _core.EdgeVector_empty(self)

    def size(self):
        return _core.EdgeVector_size(self)

    def swap(self, v):
        return _core.EdgeVector_swap(self, v)

    def begin(self):
        return _core.EdgeVector_begin(self)

    def end(self):
        return _core.EdgeVector_end(self)

    def rbegin(self):
        return _core.EdgeVector_rbegin(self)

    def rend(self):
        return _core.EdgeVector_rend(self)

    def clear(self):
        return _core.EdgeVector_clear(self)

    def get_allocator(self):
        return _core.EdgeVector_get_allocator(self)

    def pop_back(self):
        return _core.EdgeVector_pop_back(self)

    def erase(self, *args):
        return _core.EdgeVector_erase(self, *args)

    def __init__(self, *args):
        this = _core.new_EdgeVector(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _core.EdgeVector_push_back(self, x)

    def front(self):
        return _core.EdgeVector_front(self)

    def back(self):
        return _core.EdgeVector_back(self)

    def assign(self, n, x):
        return _core.EdgeVector_assign(self, n, x)

    def resize(self, *args):
        return _core.EdgeVector_resize(self, *args)

    def insert(self, *args):
        return _core.EdgeVector_insert(self, *args)

    def reserve(self, n):
        return _core.EdgeVector_reserve(self, n)

    def capacity(self):
        return _core.EdgeVector_capacity(self)
    __swig_destroy__ = _core.delete_EdgeVector
    __del__ = lambda self: None
EdgeVector_swigregister = _core.EdgeVector_swigregister
EdgeVector_swigregister(EdgeVector)

class IntVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntVector, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _core.IntVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _core.IntVector___nonzero__(self)

    def __bool__(self):
        return _core.IntVector___bool__(self)

    def __len__(self):
        return _core.IntVector___len__(self)

    def __getslice__(self, i, j):
        return _core.IntVector___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _core.IntVector___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _core.IntVector___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _core.IntVector___delitem__(self, *args)

    def __getitem__(self, *args):
        return _core.IntVector___getitem__(self, *args)

    def __setitem__(self, *args):
        return _core.IntVector___setitem__(self, *args)

    def pop(self):
        return _core.IntVector_pop(self)

    def append(self, x):
        return _core.IntVector_append(self, x)

    def empty(self):
        return _core.IntVector_empty(self)

    def size(self):
        return _core.IntVector_size(self)

    def swap(self, v):
        return _core.IntVector_swap(self, v)

    def begin(self):
        return _core.IntVector_begin(self)

    def end(self):
        return _core.IntVector_end(self)

    def rbegin(self):
        return _core.IntVector_rbegin(self)

    def rend(self):
        return _core.IntVector_rend(self)

    def clear(self):
        return _core.IntVector_clear(self)

    def get_allocator(self):
        return _core.IntVector_get_allocator(self)

    def pop_back(self):
        return _core.IntVector_pop_back(self)

    def erase(self, *args):
        return _core.IntVector_erase(self, *args)

    def __init__(self, *args):
        this = _core.new_IntVector(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _core.IntVector_push_back(self, x)

    def front(self):
        return _core.IntVector_front(self)

    def back(self):
        return _core.IntVector_back(self)

    def assign(self, n, x):
        return _core.IntVector_assign(self, n, x)

    def resize(self, *args):
        return _core.IntVector_resize(self, *args)

    def insert(self, *args):
        return _core.IntVector_insert(self, *args)

    def reserve(self, n):
        return _core.IntVector_reserve(self, n)

    def capacity(self):
        return _core.IntVector_capacity(self)
    __swig_destroy__ = _core.delete_IntVector
    __del__ = lambda self: None
IntVector_swigregister = _core.IntVector_swigregister
IntVector_swigregister(IntVector)

class IntIntPair(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntIntPair, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntIntPair, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        this = _core.new_IntIntPair(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_setmethods__["first"] = _core.IntIntPair_first_set
    __swig_getmethods__["first"] = _core.IntIntPair_first_get
    if _newclass:
        first = _swig_property(_core.IntIntPair_first_get, _core.IntIntPair_first_set)
    __swig_setmethods__["second"] = _core.IntIntPair_second_set
    __swig_getmethods__["second"] = _core.IntIntPair_second_get
    if _newclass:
        second = _swig_property(_core.IntIntPair_second_get, _core.IntIntPair_second_set)
    def __len__(self):
        return 2
    def __repr__(self):
        return str((self.first, self.second))
    def __getitem__(self, index): 
        if not (index % 2):
            return self.first
        else:
            return self.second
    def __setitem__(self, index, val):
        if not (index % 2):
            self.first = val
        else:
            self.second = val
    __swig_destroy__ = _core.delete_IntIntPair
    __del__ = lambda self: None
IntIntPair_swigregister = _core.IntIntPair_swigregister
IntIntPair_swigregister(IntIntPair)

class LongLongPair(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, LongLongPair, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, LongLongPair, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        this = _core.new_LongLongPair(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_setmethods__["first"] = _core.LongLongPair_first_set
    __swig_getmethods__["first"] = _core.LongLongPair_first_get
    if _newclass:
        first = _swig_property(_core.LongLongPair_first_get, _core.LongLongPair_first_set)
    __swig_setmethods__["second"] = _core.LongLongPair_second_set
    __swig_getmethods__["second"] = _core.LongLongPair_second_get
    if _newclass:
        second = _swig_property(_core.LongLongPair_second_get, _core.LongLongPair_second_set)
    def __len__(self):
        return 2
    def __repr__(self):
        return str((self.first, self.second))
    def __getitem__(self, index): 
        if not (index % 2):
            return self.first
        else:
            return self.second
    def __setitem__(self, index, val):
        if not (index % 2):
            self.first = val
        else:
            self.second = val
    __swig_destroy__ = _core.delete_LongLongPair
    __del__ = lambda self: None
LongLongPair_swigregister = _core.LongLongPair_swigregister
LongLongPair_swigregister(LongLongPair)

class Edge(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Edge, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Edge, name)
    __repr__ = _swig_repr
    __swig_setmethods__["next"] = _core.Edge_next_set
    __swig_getmethods__["next"] = _core.Edge_next_get
    if _newclass:
        next = _swig_property(_core.Edge_next_get, _core.Edge_next_set)
    __swig_setmethods__["u"] = _core.Edge_u_set
    __swig_getmethods__["u"] = _core.Edge_u_get
    if _newclass:
        u = _swig_property(_core.Edge_u_get, _core.Edge_u_set)
    __swig_setmethods__["v"] = _core.Edge_v_set
    __swig_getmethods__["v"] = _core.Edge_v_get
    if _newclass:
        v = _swig_property(_core.Edge_v_get, _core.Edge_v_set)
    __swig_setmethods__["flow"] = _core.Edge_flow_set
    __swig_getmethods__["flow"] = _core.Edge_flow_get
    if _newclass:
        flow = _swig_property(_core.Edge_flow_get, _core.Edge_flow_set)
    __swig_setmethods__["cost"] = _core.Edge_cost_set
    __swig_getmethods__["cost"] = _core.Edge_cost_get
    if _newclass:
        cost = _swig_property(_core.Edge_cost_get, _core.Edge_cost_set)

    def __init__(self, *args):
        this = _core.new_Edge(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _core.delete_Edge
    __del__ = lambda self: None
Edge_swigregister = _core.Edge_swigregister
Edge_swigregister(Edge)
cvar = _core.cvar

class Graph(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Graph, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Graph, name)
    __repr__ = _swig_repr
    __swig_setmethods__["cnt"] = _core.Graph_cnt_set
    __swig_getmethods__["cnt"] = _core.Graph_cnt_get
    if _newclass:
        cnt = _swig_property(_core.Graph_cnt_get, _core.Graph_cnt_set)
    __swig_setmethods__["head"] = _core.Graph_head_set
    __swig_getmethods__["head"] = _core.Graph_head_get
    if _newclass:
        head = _swig_property(_core.Graph_head_get, _core.Graph_head_set)
    __swig_setmethods__["edges"] = _core.Graph_edges_set
    __swig_getmethods__["edges"] = _core.Graph_edges_get
    if _newclass:
        edges = _swig_property(_core.Graph_edges_get, _core.Graph_edges_set)

    def __init__(self, n):
        this = _core.new_Graph(n)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _core.delete_Graph
    __del__ = lambda self: None

    def add_edge(self, u, v, flow, cost=0):
        return _core.Graph_add_edge(self, u, v, flow, cost)
Graph_swigregister = _core.Graph_swigregister
Graph_swigregister(Graph)

class BaseNetwork(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, BaseNetwork, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, BaseNetwork, name)
    __repr__ = _swig_repr
    __swig_setmethods__["n"] = _core.BaseNetwork_n_set
    __swig_getmethods__["n"] = _core.BaseNetwork_n_get
    if _newclass:
        n = _swig_property(_core.BaseNetwork_n_get, _core.BaseNetwork_n_set)
    __swig_setmethods__["dist"] = _core.BaseNetwork_dist_set
    __swig_getmethods__["dist"] = _core.BaseNetwork_dist_get
    if _newclass:
        dist = _swig_property(_core.BaseNetwork_dist_get, _core.BaseNetwork_dist_set)
    __swig_setmethods__["graph"] = _core.BaseNetwork_graph_set
    __swig_getmethods__["graph"] = _core.BaseNetwork_graph_get
    if _newclass:
        graph = _swig_property(_core.BaseNetwork_graph_get, _core.BaseNetwork_graph_set)

    def __init__(self, n):
        this = _core.new_BaseNetwork(n)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _core.delete_BaseNetwork
    __del__ = lambda self: None
BaseNetwork_swigregister = _core.BaseNetwork_swigregister
BaseNetwork_swigregister(BaseNetwork)

class MaximumFlow(BaseNetwork):
    __swig_setmethods__ = {}
    for _s in [BaseNetwork]:
        __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, MaximumFlow, name, value)
    __swig_getmethods__ = {}
    for _s in [BaseNetwork]:
        __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, MaximumFlow, name)
    __repr__ = _swig_repr
    __swig_setmethods__["cur"] = _core.MaximumFlow_cur_set
    __swig_getmethods__["cur"] = _core.MaximumFlow_cur_get
    if _newclass:
        cur = _swig_property(_core.MaximumFlow_cur_get, _core.MaximumFlow_cur_set)

    def __init__(self, n):
        this = _core.new_MaximumFlow(n)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _core.delete_MaximumFlow
    __del__ = lambda self: None

    def bfs(self, S, T):
        return _core.MaximumFlow_bfs(self, S, T)

    def dfs(self, *args):
        return _core.MaximumFlow_dfs(self, *args)

    def run(self, S, T):
        return _core.MaximumFlow_run(self, S, T)
MaximumFlow_swigregister = _core.MaximumFlow_swigregister
MaximumFlow_swigregister(MaximumFlow)

class MinimumCostFlow(BaseNetwork):
    __swig_setmethods__ = {}
    for _s in [BaseNetwork]:
        __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, MinimumCostFlow, name, value)
    __swig_getmethods__ = {}
    for _s in [BaseNetwork]:
        __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, MinimumCostFlow, name)
    __repr__ = _swig_repr
    __swig_setmethods__["pre"] = _core.MinimumCostFlow_pre_set
    __swig_getmethods__["pre"] = _core.MinimumCostFlow_pre_get
    if _newclass:
        pre = _swig_property(_core.MinimumCostFlow_pre_get, _core.MinimumCostFlow_pre_set)
    __swig_setmethods__["low"] = _core.MinimumCostFlow_low_set
    __swig_getmethods__["low"] = _core.MinimumCostFlow_low_get
    if _newclass:
        low = _swig_property(_core.MinimumCostFlow_low_get, _core.MinimumCostFlow_low_set)
    __swig_setmethods__["vis"] = _core.MinimumCostFlow_vis_set
    __swig_getmethods__["vis"] = _core.MinimumCostFlow_vis_get
    if _newclass:
        vis = _swig_property(_core.MinimumCostFlow_vis_get, _core.MinimumCostFlow_vis_set)
    __swig_setmethods__["clk"] = _core.MinimumCostFlow_clk_set
    __swig_getmethods__["clk"] = _core.MinimumCostFlow_clk_get
    if _newclass:
        clk = _swig_property(_core.MinimumCostFlow_clk_get, _core.MinimumCostFlow_clk_set)

    def __init__(self, n):
        this = _core.new_MinimumCostFlow(n)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _core.delete_MinimumCostFlow
    __del__ = lambda self: None

    def bfs(self, S, T):
        return _core.MinimumCostFlow_bfs(self, S, T)

    def run(self, S, T):
        return _core.MinimumCostFlow_run(self, S, T)
MinimumCostFlow_swigregister = _core.MinimumCostFlow_swigregister
MinimumCostFlow_swigregister(MinimumCostFlow)

# This file is compatible with both classic and new-style classes.


